@@ -45,16 +45,9 @@ def option_question_callback(opt, desc):
     if not ok: sys.exit(0)
     return ans
 
-def start(auto=True, test_depends=False, test_unknown=False):
+def start(language, auto=True, test_depends=False, test_unknown=False):
     try:
-        log.info("Initializing. Please wait...")
         core =  CoreInstall(MODE_INSTALLER)
-        core.init()
-        
-        if test_unknown:
-            core.distro_name = 'unknown'
-            core.distro = 0
-            core.distro_version = 0
     
         if core.running_as_root():
             log.error("You are running the installer as root. It is highly recommended that you run the installer as")
@@ -75,12 +68,33 @@ def start(auto=True, test_depends=False, test_unknown=False):
             log.info("Automatic mode will install the full HPLIP solution with the most common options.")
             log.info("Custom mode allows you to chose installation options to fit specific requirements.")
             
-            ok, choice = tui.enter_choice("\nPlease choose the installation mode (a=automatic*, c=custom, q=quit) : ", 
-                ['a', 'c'], 'a')
+            if os.getenv('DISPLAY') and utils.find_browser() is not None:
+                ok, choice = tui.enter_choice("\nPlease choose the installation mode (a=automatic*, c=custom, w=web installer, q=quit) : ", 
+                    ['a', 'c', 'w'], 'a')
+            else:
+                ok, choice = tui.enter_choice("\nPlease choose the installation mode (a=automatic*, c=custom, q=quit) : ", 
+                    ['a', 'c'], 'a')
+                    
             if not ok: sys.exit(0)
+            
             if choice == 'a':
                 auto = True
+                
+            elif choice == 'w':
+                import web_install
+                log.debug("Starting web browser installer...")
+                web_install.start(language)
+                return
 
+        log.info("\nInitializing. Please wait...")
+        core.init()
+        
+        if test_unknown:
+            core.distro_name = 'unknown'
+            core.distro = 0
+            core.distro_version = 0
+        
+        
         #
         # HPLIP vs. HPIJS INSTALLATION
         #
@@ -264,7 +278,6 @@ def start(auto=True, test_depends=False, test_unknown=False):
                 if core.distro_version_int == 0:
                     core.distro_version = DISTRO_VER_UNKNOWN
                     core.distro_version_supported = False
-
                 else:
                     core.distro_version = versions[core.distro_version_int - 1]
                     core.distro_version_supported = core.get_ver_data('supported', False)
@@ -610,10 +623,19 @@ def start(auto=True, test_depends=False, test_unknown=False):
                     log.warn("Continuing to run installer - this installation should overwrite the previous one.")
 
 
+            #
+            # POST-DEPEND
+            #
+            
+            tui.title("RUNNING POST-PACKAGE COMMANDS")
+            core.run_post_depend(progress_callback)
+            
+            
             # 
             # DEPENDENCIES RE-CHECK
             #
-
+            
+            tui.title("RE-CHECKING DEPENDENCIES")
             core.check_dependencies()
 
             num_req_missing = 0
@@ -637,14 +659,13 @@ def start(auto=True, test_depends=False, test_unknown=False):
                     core.selected_options[opt] = False
                 else:
                     log.warn("An optional dependency '%s (%s)' is still missing." % (depend, desc))
-                    log.warn("Some features may not function as expected.")                
+                    log.warn("Some features may not function as expected.") 
+     
+            
+            if not num_opt_missing and not num_req_missing:
+                log.info("OK")
 
 
-            #
-            # POST-DEPEND
-            #
-            tui.title("RUNNING POST-PACKAGE COMMANDS")
-            core.run_post_depend(progress_callback)
 
         #
         # INSTALL LOCATION
